// Updates: 2002.07.02


/*
 * Programme tourant sur le PC permettant de calibrer le RCX.
 */



import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import java.io.*;
import josx.rcxcomm.*;


public class CalibPC extends JFrame implements ActionListener {
  JTextField tfMotorA, tfMotorC, tfTimeOneMeter, tfTimeRotate, tfDelay;
  String[] buttonLabels = {"Move forwards", "Move backwards", "Spin left", "Spin right", "360 left", "360 right", "10x36 left", "10x36 right", "Move 1 m. for.", "Move 1 m. back."};
  String powerMotorA, powerMotorC, timeOneMeter, timeOneRotation, delay;
  DataOutputStream out;  
  Properties props;
  String propsFile;
  
  public CalibPC(String propsFile) {
    super("   RCX Calibration     -     " + propsFile);
    this.propsFile = propsFile;
    LookAndFeelMenu lafm = new LookAndFeelMenu(this, true);
    lafm.addPlaf("Kunststoff", "com.incors.plaf.kunststoff.KunststoffLookAndFeel");
    lafm.setLaf("Kunststoff");       
    loadParams();
    JPanel p = new JPanel();
    p.setLayout(new GridLayout(1, 2));
    JPanel p1 = new JPanel();
    p1.setBorder(BorderFactory.createTitledBorder("   Values   "));
    p1.setLayout(new GridLayout(5, 2));
    p1.add(new JLabel("   Motor A:"));
    tfMotorA = new JTextField(powerMotorA);
    p1.add(tfMotorA);
    p1.add(new JLabel("   Motor C:"));
    tfMotorC = new JTextField(powerMotorC);
    p1.add(tfMotorC); 
    p1.add(new JLabel("   Time one meter:"));
    tfTimeOneMeter = new JTextField(timeOneMeter);
    p1.add(tfTimeOneMeter); 
    p1.add(new JLabel("   Time one rotation:"));
    tfTimeRotate = new JTextField(timeOneRotation);
    p1.add(tfTimeRotate); 
    p1.add(new JLabel("   Delay:"));
    tfDelay = new JTextField(delay);
    p1.add(tfDelay);
    p.add(p1);
    JPanel p2 = new JPanel();
    p2.setBorder(BorderFactory.createTitledBorder("   Tests  "));
    p2.setLayout(new GridLayout(5, 2));
    for (int i = 0; i < buttonLabels.length; i++) {
      JButton b = new JButton(buttonLabels[i]);
      b.addActionListener(this);
      p2.add(b);
    }
    p.add(p2);     
    
    getContentPane().add(p);

    try {
      RCXPort port = new RCXPort("usb");
      out = new DataOutputStream(port.getOutputStream());      
    }
    catch (IOException e) {
      e.printStackTrace();  
      System.exit(0);
    }      

    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        saveParams();
        System.exit(0);
      }
    });
    setSize(570, 200);
    Dimension d1 = getToolkit().getScreenSize();
    Dimension d2 = this.getSize();     
    setLocation(d1.width/2 - d2.width/2, d1.height/2 - d2.height/2);
    SwingUtilities.updateComponentTreeUI(this);
    show();
  }


  public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();
    if (cmd.equals("Move forwards"))  send(1);
    if (cmd.equals("Move backwards")) send(2);
    if (cmd.equals("Spin left"))      send(3);
    if (cmd.equals("Spin right"))     send(4);
    if (cmd.equals("360 left"))       send(5);
    if (cmd.equals("360 right"))      send(6);
    if (cmd.equals("10x36 left"))     send(7);
    if (cmd.equals("10x36 right"))    send(8);
    if (cmd.equals("Move 1 m. for."))   send(9);
    if (cmd.equals("Move 1 m. back."))  send(10);
  }  


  private void send(int msg) {
    try {
      out.writeShort(Short.parseShort(tfMotorA.getText()));
      out.writeShort(Short.parseShort(tfMotorC.getText()));
      out.writeInt((int)(100000.0*Double.parseDouble(tfTimeOneMeter.getText())));
      out.writeInt((int)(100000.0*Double.parseDouble(tfTimeRotate.getText())));
      out.writeShort(Short.parseShort(tfDelay.getText()));
      out.writeShort((short)msg);
      out.flush();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

    
  public void loadParams() {
    props = new Properties();
    try {
      props.load(new FileInputStream(propsFile));
    }
    catch (IOException e) {
    }
    powerMotorA = props.getProperty("powerMotorA");
    powerMotorC = props.getProperty("powerMotorC");     
    timeOneMeter = props.getProperty("timeOneMeter"); 
    timeOneRotation = props.getProperty("timeOneRotation"); 
    delay = props.getProperty("delay"); 
  }  


  public void saveParams() {
    props.setProperty("powerMotorA", tfMotorA.getText());
    props.setProperty("powerMotorC", tfMotorC.getText());
    props.setProperty("timeOneMeter", tfTimeOneMeter.getText());
    props.setProperty("timeOneRotation", tfTimeRotate.getText());
    props.setProperty("delay", tfDelay.getText());
    try {
      props.store(new FileOutputStream(propsFile), null);
    }
    catch (IOException e) {
      System.out.println("Erreur: " + e);
    }
  }
  

  public static void main(String[] args) {
    if (args.length == 0) new CalibPC("CalibPC.properties");
    else new CalibPC(args[0] + ".properties");
  }
}