// Updates: 2002.06.26


/* 
 * Cette classe est responsable de l'apparence des composants de
 * l'interface utilisateur.
 */ 


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.util.*;


public class LookAndFeelMenu implements ActionListener {
  JMenu menu;
  ArrayList names = new ArrayList();
  ArrayList classNames = new ArrayList();
  JFrame frame; 

  
  /*
   * Cre un menu Look & Feel. Si defaultLaf vaut true, alors tous les L&F
   * disponibles par dfaut sont rajouts. 
   */  
  public LookAndFeelMenu(JFrame frame, boolean defaultLaf) {
    this.frame = frame;
    menu = new JMenu("Look & Feel");
    if (defaultLaf) addDefaultLaf();
  }
  
  
  /*
   * Recherche tous les Look & Feel disponibles par dfaut et les ajoute
   * au menu.
   */
  public void addDefaultLaf() {
    UIManager.LookAndFeelInfo[] lafi = UIManager.getInstalledLookAndFeels();
    for (int i = 0; i < lafi.length; i++) {
      names.add(lafi[i].getName());
      classNames.add(lafi[i].getClassName());
      JMenuItem mi = new JMenuItem(lafi[i].getName());
      mi.addActionListener(this);
      menu.add(mi);
    }
  }

  
  /*
   * Retourne le menu Look & Feel.
   */ 
  public JMenu getMenu() {
    return menu; 
  }
  
  
  /*
   * Ajoute un Look & Feel supplmentaire. 
   */
  public void addPlaf(String name, String className) {
    names.add(name);
    classNames.add(className);    
    JMenuItem mi = new JMenuItem(name);
    mi.addActionListener(this);
    menu.add(mi);    
  }


  /*
   * Permet de spcifier le Look & Feel courant par son index, puis de 
   * mettre  jour tous les composants.
   */
  public void setLaf(int i) {
    try {
      UIManager.setLookAndFeel((String)classNames.get(i));
    }
    catch (Exception ex) {
      System.out.println(ex); 
    }
    SwingUtilities.updateComponentTreeUI(frame);
  }           


  /*
   * Permet de spcifier le Look & Feel courant par son nom, puis de 
   * mettre  jour tous les composants.
   */  
  public void setLaf(String name) {
    for (int i = 0; i < names.size(); i++) {
      if (name.equals((String)names.get(i))) setLaf(i);
    }
  }  
    
  
  public void actionPerformed(ActionEvent e) {
    for (int i = 0; i < names.size(); i++) {          
      if (e.getActionCommand().equals((String)names.get(i))) setLaf(i);
    }
  }
}