// Updates: 2002.06.03


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;


public class AI {
  final int DELTA_RCX_CYLINDER = 200;
  final int DELTA_RCX_CYLINDER_B = 150;
  final int CYLINDER_ZONE_RADIUS = 250;
  final int CYLINDER_ZONE_RADIUS_B = 180;
  final int DELTA_RCX_GOAL = 70;
  final int GOAL_ZONE_RADIUS = 120;
  final int DELTA_RCX_DOOR = 120;
  final int DOOR_CHANNEL_WIDTH = 90;
  final int DOOR_CHANNEL_HEIGHT = 280; 
  final int HOME_ZONE_RADIUS = 50;
  final Point HOME = new Point(100, 125);
  
  boolean goingToHome, goingToGoal, justDroppedACylinder, finished;
  Positions pos;
  Point currentCylinder;
  Point currentGoal;
  int nearestCylinder = -2;
  
  
  public AI() {
  }
  
  
  public String next(Positions positions) {
    pos = positions;
    if (finished) return "" + GC.FINISHED;
    if (justDroppedACylinder) {
      justDroppedACylinder = false;
      goingToGoal = false;
      changeCurrentCylinder();      
    }
    if (nearestCylinder == -2) changeCurrentCylinder();
    if (nearestCylinder == -1) goingToHome = true;
    if (goingToHome) return pathToHome();
    else if (goingToGoal) return pathToGoal();
    else return pathToCylinder();
  }
  
  
  public int getCurrentCylinderIndex() {
    return nearestCylinder;
  }
  
  
  public void changeCurrentCylinder() {
    nearestCylinder = getNearestCylinder();
    if (nearestCylinder == 0) {
      currentCylinder = new Point(pos.blueCylinder.x, pos.blueCylinder.y);
      currentGoal = new Point(pos.blueGoal.x, pos.blueGoal.y);
    }
    if (nearestCylinder == 1) {
      currentCylinder = new Point(pos.greenCylinder.x, pos.greenCylinder.y);
      currentGoal = new Point(pos.greenGoal.x, pos.greenGoal.y);
    }
    if (nearestCylinder == 2) {
      currentCylinder = new Point(pos.redCylinder.x, pos.redCylinder.y);
      currentGoal = new Point(pos.redGoal.x, pos.redGoal.y);
    }
  }
 
  
  private int getNearestCylinder() {
    int distBlue = distance(pos.blueCylinder, pos.blueGoal);
    int distGreen = distance(pos.greenCylinder, pos.greenGoal);
    int distRed = distance(pos.redCylinder, pos.redGoal);
    if ((distBlue == distGreen) && (distBlue == distRed)) return -1;
    if ((distBlue <= distGreen) && (distBlue <= distRed)) return 0;
    if ((distGreen <= distBlue) && (distGreen <= distRed)) return 1;
    return 2;
  }


  private int distance(Point cylinder, Point goal) {
    int cx = cylinder.x;
    int cy = cylinder.y;
    int gx = goal.x;
    int gy = goal.y;
    int hdx = pos.horizontalDoor.x;
    int hdy = pos.horizontalDoor.y;
    int vdx = pos.verticalDoor.x;
    int vdy = pos.verticalDoor.y;    
    if (cy < gy+50) {
      return 10000;
    }
    else if (isInZone0(cylinder)) { 
      return (int)Math.sqrt(cx*cx+cy*cy);
    }
    else if (isInZone1(cylinder)) {
      return 1000+(int)Math.sqrt((cx-hdx)*(cx-hdx)+(cy-hdy)*(cy-hdy));
    }
    else if (isInZone2(cylinder)) {
      return 2000+(int)Math.sqrt((cx-vdx)*(cx-vdx)+(cy-vdy)*(cy-vdy));
    }
    else return 10000;
  }  
  
  
  private boolean isInZone0(Point p) {
    return (p.y <= pos.horizontalDoor.y);
  }
  
  
  private boolean isInZone1(Point p) {
    return ((p.y >= pos.horizontalDoor.y) && (p.x < pos.verticalDoor.x));
  }  
  
  
  private boolean isInZone2(Point p) {
    return ((p.y >= pos.horizontalDoor.y) && (p.x > pos.verticalDoor.x));
  }
  
  
  private boolean isInZone0(Triplet p) {
    return (p.y <= pos.horizontalDoor.y);
  }
  
  
  private boolean isInZone1(Triplet p) {
    return ((p.y >= pos.horizontalDoor.y) && (p.x < pos.verticalDoor.x));
  }  
  
  
  private boolean isInZone2(Triplet p) {
    return ((p.y >= pos.horizontalDoor.y) && (p.x > pos.verticalDoor.x));
  }     


  private boolean isInZone1or2(Point p) {
    return (isInZone1(p) || isInZone2(p));
  } 
  
  
  private boolean isInHorizontalDoorZone(Triplet p) {
    return (p.x > pos.horizontalDoor.x-DOOR_CHANNEL_WIDTH/2) &&
           (p.x < pos.horizontalDoor.x+DOOR_CHANNEL_WIDTH/2) &&
           (p.y > pos.horizontalDoor.y-DOOR_CHANNEL_HEIGHT/2) &&
           (p.y < pos.horizontalDoor.y+DOOR_CHANNEL_HEIGHT/2);
  }
   
   
  private boolean isInVerticalDoorZone(Triplet p) {
    return (p.x > pos.verticalDoor.x-DOOR_CHANNEL_HEIGHT/2) &&
           (p.x < pos.verticalDoor.x+DOOR_CHANNEL_HEIGHT/2) &&
           (p.y > pos.verticalDoor.y-DOOR_CHANNEL_WIDTH/2) &&
           (p.y < pos.verticalDoor.y+DOOR_CHANNEL_WIDTH/2);
  }
            
  
  private boolean isInHorizontalDoorZone0(Triplet p) {
    return isInHorizontalDoorZone(p) && isInZone0(p);
  }
  
  
  private boolean isInHorizontalDoorZone1(Triplet p) {
    return isInHorizontalDoorZone(p) && isInZone1(p);
  }
    

  private boolean isInVerticalDoorZone1(Triplet p) {
    return isInVerticalDoorZone(p) && isInZone1(p);
  }


  private boolean isInVerticalDoorZone2(Triplet p) {
    return isInVerticalDoorZone(p) && isInZone2(p);
  }
  

  private boolean isInCurrentCylinderZone(Triplet p) {
    int cx = currentCylinder.x;
    int cy = currentCylinder.y;
    return Math.sqrt((p.x-cx)*(p.x-cx)+(p.y-cy)*(p.y-cy)) <= CYLINDER_ZONE_RADIUS;
  }
   
   
  private boolean isInCurrentCylinderZoneB(Triplet p) {
    int cx = currentCylinder.x;
    int cy = currentCylinder.y;
    return Math.sqrt((p.x-cx)*(p.x-cx)+(p.y-cy)*(p.y-cy)) <= CYLINDER_ZONE_RADIUS_B;
  }
     
   
  private boolean isInCurrentGoalZone(Triplet p) {
    int gx = currentGoal.x;
    int gy = currentGoal.y;
    return Math.sqrt((p.x-gx)*(p.x-gx)+(p.y-gy)*(p.y-gy)) <= GOAL_ZONE_RADIUS;
  }
  

  private boolean isInHomeZone(Triplet p) {
    int hx = HOME.x;
    int hy = HOME.y;
    return Math.sqrt((p.x-hx)*(p.x-hx)+(p.y-hy)*(p.y-hy)) <= HOME_ZONE_RADIUS;
  }         
  
  private String pathToCylinder() {
    if (isInHorizontalDoorZone0(pos.rcx)) {
      return gotoPoint(new Point(pos.horizontalDoor.x,pos.horizontalDoor.y+DELTA_RCX_DOOR), 0);
    }
    if (isInVerticalDoorZone1(pos.rcx)) {
      return gotoPoint(new Point(pos.verticalDoor.x+DELTA_RCX_DOOR,pos.verticalDoor.y), 0);
    }
    if (isInCurrentCylinderZoneB(pos.rcx)) {
      goingToGoal = true;
      return ""+GC.CMD_TAKE_CYL;
    }
    if (isInCurrentCylinderZone(pos.rcx)) {
      return gotoPoint(currentCylinder, DELTA_RCX_CYLINDER_B);
    }    
    if (isInZone0(pos.rcx) && isInZone0(currentCylinder)) {
      return gotoPoint(currentCylinder, DELTA_RCX_CYLINDER);
    }
    if (isInZone0(pos.rcx) && isInZone1or2(currentCylinder)) {
      return gotoPoint(new Point(pos.horizontalDoor.x,pos.horizontalDoor.y-DELTA_RCX_DOOR), 0);
    }
    if (isInZone1(pos.rcx) && isInZone1(currentCylinder)) {
      return gotoPoint(currentCylinder, DELTA_RCX_CYLINDER);
    }
    if (isInZone1(pos.rcx) && isInZone2(currentCylinder)) {
      return gotoPoint(new Point(pos.verticalDoor.x-DELTA_RCX_DOOR,pos.verticalDoor.y), 0);
    }
    if (isInZone2(pos.rcx)) {
      return gotoPoint(currentCylinder, DELTA_RCX_CYLINDER);
    }            
    return "0 0";    
  }
   
   
  private String pathToGoal() {
    if (isInHorizontalDoorZone1(pos.rcx)) {
      return gotoPoint(new Point(pos.horizontalDoor.x,pos.horizontalDoor.y-DELTA_RCX_DOOR), 0);
    }  
    if (isInVerticalDoorZone2(pos.rcx)) {
      return gotoPoint(new Point(pos.verticalDoor.x-DELTA_RCX_DOOR,pos.verticalDoor.y), 0);
    }      
    if (isInCurrentGoalZone(pos.rcx)) {
      justDroppedACylinder = true;
      return ""+GC.CMD_DROP_CYL;
    }      
    if (isInZone0(pos.rcx)) {
      return gotoPoint(currentGoal, DELTA_RCX_GOAL);
    }
    if (isInZone1(pos.rcx)) {
      return gotoPoint(new Point(pos.horizontalDoor.x,pos.horizontalDoor.y+DELTA_RCX_DOOR), 0);
    } 
    if (isInZone2(pos.rcx)) {
      return gotoPoint(new Point(pos.verticalDoor.x+DELTA_RCX_DOOR,pos.verticalDoor.y), 0);
    }        
    return "0 0";    
  }    
  
  
  private String pathToHome() {
    if (isInHorizontalDoorZone0(pos.rcx)) {
      return gotoPoint(HOME, 0);
    }
    if (isInHomeZone(pos.rcx)) {
      finished = true;
      return headToNorth();
    }
    return gotoPoint(new Point(pos.horizontalDoor.x,pos.horizontalDoor.y-DELTA_RCX_DOOR), 0);
  }
  
  
  private String gotoPoint(Point p, int delta) {
    double dx = p.getX()-(double)pos.rcx.x;
    double dy = p.getY()-(double)pos.rcx.y;
    double alpha = Math.toDegrees(Math.atan(dy/dx))-90;
    if (dx < 0) alpha += 180;
    int angle = (int)alpha-pos.rcx.a;
    angle = angle + 720;
    angle = angle % 360;
    if (angle > 180) angle = angle - 360;
    int dist = (int)Math.sqrt(dx*dx+dy*dy)-delta;
    return angle + " " + dist;
  }
  
  
  private String headToNorth() {
    return -pos.rcx.a + " " + 0;
  }  
}  