/* EHRLER FREDERIC - MICHEL DERIAZ
 * ------------------------------------------------------------------
 * Cette classe contient tout les outils d'image pour appliquer des 
 * traitement a l'image
 * 
 * operateur disponible:
 * - correction gamma
 * - selection de bande de l'image
 * - binarization
 * - passage en noire et blanc
 * - changement de gamme dynamique
 * - filtre moyenneur
 * - operation de contour
 * - selection de composante de couleur
 *
 */


import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.awt.image.*;
import java.awt.image.renderable.*;
import javax.media.jai.*;
import com.sun.media.jai.codec.*;
import java.awt.color.*;

public class ImageTools{


	public static BufferedImage applyBorder(BufferedImage buffImgSource,int borderSize, int value){
		BufferedImage buffImgDest;
		
		PlanarImage image = PlanarImage.wrapRenderedImage(buffImgSource);
		Rectangle rect = new Rectangle(borderSize, borderSize,320-2*borderSize, 240-2*borderSize);
    WritableRaster raster = (WritableRaster)image.getExtendedData(rect,null);
    double[] border = new double[borderSize] ;
    for (int i=0; i<borderSize; i++){
    	border[i] = (double)value;
    }
    BorderExtenderConstant BEC = new BorderExtenderConstant(border);
    BEC.extend(raster,image);
    buffImgDest = image.getAsBufferedImage();
    
    return buffImgDest;
 	}
	
// OPERATEUR DE GAMMA CORRECTION
// ------------------------------------------------------------------------
	public static BufferedImage correctGamma(BufferedImage buffImgSource, double gamma){
		BufferedImage buffImgDest;
		int[][] tableData = new int[3][256];
		for (int i=0; i<256; i++){
			tableData[0][i] = (int)(255 * Math.pow(i,(1/gamma))/ Math.pow(255,(1/gamma)));
			tableData[1][i] = (int)(255 * Math.pow(i,(1/gamma))/ Math.pow(255,(1/gamma)));
			tableData[2][i] = (int)(255 * Math.pow(i,(1/gamma))/ Math.pow(255,(1/gamma)));
		}
		
		LookupTableJAI table = new LookupTableJAI(tableData);
		PlanarImage imgDest = JAI.create("lookup",buffImgSource,table);
		buffImgDest = imgDest.getAsBufferedImage();
			
		return buffImgDest;
	}


/* OPERATEUR DE SELECTION DE BANDE DE COULEURS
 * ------------------------------------------------------------------------
 */
	public static BufferedImage getBand(BufferedImage buffImgSource, int band){
		//BufferedImage buffImgSource;
		BufferedImage buffImgDest;
		
		int[] bandIndices = new int[3];
		bandIndices[0] = band;
		bandIndices[1] = band;
		bandIndices[2] = band;
		
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(buffImgSource);
		pb.add(bandIndices);
		//RenderedImage dst = (RenderedImage)JAI.create("bandSelect",pb);
		RenderedOp dst = JAI.create("bandSelect",pb);
		PlanarImage imgDest = dst.createInstance();
		
		buffImgDest = imgDest.getAsBufferedImage();
		
		return buffImgDest;
	}
	

// OPERATEUR DE SELECTION DE BINARIZATION DE L'IMAGE	
// ------------------------------------------------------------------------
	public static BufferedImage binarize(BufferedImage buffImgSource, double seuil){
		BufferedImage buffImgDest;
		ParameterBlock pb = new ParameterBlock();
	
		pb = new ParameterBlock();
		pb.addSource(buffImgSource);
		pb.add(seuil);
		PlanarImage imgDest = (PlanarImage)JAI.create("binarize",pb);
		buffImgDest = imgDest.getAsBufferedImage();
		
		return buffImgDest;
	}
	
// OPERATEUR DE NOIRE ET BLANC 
// ------------------------------------------------------------------------
	public static BufferedImage noireBlanc(BufferedImage source) {
     	BufferedImageOp op = new ColorConvertOp(
       	ColorSpace.getInstance(ColorSpace.CS_GRAY), null);
    	return op.filter(source, null);
	}
	
// OPERATEUR DE LINEARISATION DE LA BANDE DE GRAYSCALE
// ------------------------------------------------------------------------
	public static BufferedImage rescale(BufferedImage buffImgSource){
		BufferedImage buffImgDest;
		ROI roi = new ROI(buffImgSource);
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(buffImgSource);
		pb.add(roi);
		pb.add(1);
		pb.add(1);
		RenderedOp op = JAI.create("extrema", pb);
		double[][] extrema = (double[][])op.getProperty("extrema");
		//System.out.println("extrema.size := "+extrema.length+" ; "+extrema[0].length);
		//System.out.println("extrema[0][0] := "+extrema[0][0]+" extrema[1][0] "+extrema[1][0]);
		
		double max = extrema[0][0];
		double min = extrema[1][0];
		double[] scale = new double[] {255.0/(max-min)};
		double[] offset = new double[] {255.0*min/(min-max)};
		//double[] scale = new double[] {255.0/(max)};
		//double[] offset = new double[] {0};
		
		pb = new ParameterBlock();
		pb.addSource(buffImgSource);
		pb.add(scale);
		pb.add(offset);
		PlanarImage imgDest = (PlanarImage)JAI.create("rescale",pb);
		buffImgDest = imgDest.getAsBufferedImage();
	
		return buffImgDest;
	}
	
// OPERATEUR DE CONVOLUTION (FILTRAGE PASSE-BAS)
// ------------------------------------------------------------------------
	public static BufferedImage filter(BufferedImage buffImgSource, int tailleFiltre){
		BufferedImage buffImgDest;
		KernelJAI kernel;
		int size = tailleFiltre*tailleFiltre;
		float[] KernelData = new float[size];
		float filterValue = 1.0F/(size);
		
		for (int i=0;i<size;i++){
			KernelData[i]=filterValue;
		}
		
		kernel = new KernelJAI(tailleFiltre,tailleFiltre,1,1, KernelData);
		
		PlanarImage im1 = JAI.create("convolve", buffImgSource, kernel);
		buffImgDest = im1.getAsBufferedImage();	
			
		return buffImgDest;		
	}
	
	
// OPERATEUR DE RECHERCHE DE CONTOUR
// ------------------------------------------------------------------------
	public static BufferedImage Contour(BufferedImage buffImgSource){
		BufferedImage buffImgDest;
		KernelJAI kernel;
		float[] KernelData = { 	1.0F/9.0F, 1.0F/9.0F, 1.0F/9.0F,
								1.0F/9.0F, 1.0F/9.0F, 1.0F/9.0F,
								1.0F/9.0F, 1.0F/9.0F, 1.0F/9.0F
							};
		kernel = new KernelJAI(3,3,1,1, KernelData);
		
		PlanarImage im1 = JAI.create("convolve", buffImgSource, kernel);
		buffImgDest = im1.getAsBufferedImage();		
		
		return buffImgDest;	
	}
	
	
	
/* OPERATEUR SPECIAL
 * ------------------------------------------------------------------------
 * cette operateur va effectuer un seuillage special, pour que la selection
 * d'une couleurs se fasse, il faudra que la diffrence entre les couleur
 * soit plus grande que le parametre tolerance et que cette couleur elle 
 * meme soit plus grand qu'un certain seuil min
 */

	public static BufferedImage selectColorOperator(BufferedImage buffImgSource,  int tolerance, int seuilMin) {
    int w = buffImgSource.getWidth(null);
    int h = buffImgSource.getHeight(null);
    BufferedImage buffImgDest = new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);
    for (int i = 0; i < w; i++) {
      for (int j = 0; j < h; j++) {
        int pixelColor = buffImgSource.getRGB(i, j);
        int red   = (int)((pixelColor >> 16) & 0xff); 
	  		int green = (int)((pixelColor >> 8 ) & 0xff);
   			int blue  = (int)((pixelColor      ) & 0xff); 
   			if ((red>green+tolerance)&&(red>blue+tolerance)&&(red>seuilMin)){
					red = 0xff; 
					green = 0;
					blue = 0;
				} else {
					if ((green>red+tolerance)&&(green>blue+tolerance)&&(green>seuilMin)){
						red = 0; 
						green = 0xff;
						blue = 0;
					} else {
						if ((blue>green+tolerance)&&(blue>red+tolerance)&&(blue>seuilMin)){
							red = 0; 
							green = 0;
							blue = 0xff;
						} else {
							red = 0xff; 
							green = 0xff;
							blue = 0xff;
						}
					}
				}
        buffImgDest.setRGB(i, j, (0xff << 24) | (red << 16) | (green << 8) | blue);
      }
    }
    return buffImgDest;
  }
  
  
  /* OPERATEUR SPECIAL
 * ------------------------------------------------------------------------
 */

	public static BufferedImage selectColor(BufferedImage buffImgSource, int noColor, int seuilMin, int seuilMax) {
    int w = buffImgSource.getWidth(null);
    int h = buffImgSource.getHeight(null);
    int[] colorValue = new int[3];
    int[] otherColor = new int[2];
    BufferedImage buffImgDest = new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);
    System.out.println("test de la couleur no "+noColor+" avec seuil min: "+seuilMin+", seuil max: "+seuilMax);
    if (noColor==0){
    	otherColor[0] = 1;
    	otherColor[1] = 2;
    }
    if (noColor==1){
    	otherColor[0] = 0;
    	otherColor[1] = 2;
    }
    if (noColor==2){
    	otherColor[0] = 0;
    	otherColor[1] = 1;
    }
    	
    for (int i = 0; i < w; i++) {
      for (int j = 0; j < h; j++) {
        int pixelColor = buffImgSource.getRGB(i, j);
        colorValue[0] = (int)((pixelColor >> 16) & 0xff); 
	  		colorValue[1] = (int)((pixelColor >> 8 ) & 0xff);
   			colorValue[2] = (int)((pixelColor      ) & 0xff); 
   			
   			//if ((rouge>seuilMin)&&(bleu<seuilMax)&&(vert>seuilMax)){
				
				if ((colorValue[noColor]>seuilMin)&&(colorValue[otherColor[0]]<seuilMax)&&(colorValue[otherColor[1]]<seuilMax)){
					colorValue[noColor]=0x0ff;
					colorValue[otherColor[0]]=0;
					colorValue[otherColor[1]]=0;
				} else {
						colorValue[0] = 0; 
						colorValue[1] = 0;
						colorValue[2] = 0;
				}
        buffImgDest.setRGB(i, j, (0xff << 24) | (colorValue[0] << 16) | (colorValue[1] << 8) | colorValue[2]);
      }
    }
    return buffImgDest;
  }
		
	
	
	
// de selection de totues les couleurs
// ------------------------------------------------------------------------
	public static BufferedImage selectAllColor(BufferedImage buffImgSource, int seuilMin){
		BufferedImage buffImgDest;
		int[][] tableData = new int[3][256];
		for (int i=0; i<256; i++){
			if (i<seuilMin){
				tableData[0][i] = 0;
				tableData[1][i] = 0;
				tableData[2][i] = 0;
			} else {
				tableData[0][i] = 255;
				tableData[1][i] = 255;
				tableData[2][i] = 255;
				
				//tableData[0][i] = 255;
				//tableData[1][i] = 0;
				//tableData[2][i] = 0;
			}
		}
		
		LookupTableJAI table = new LookupTableJAI(tableData);
		PlanarImage imgDest = JAI.create("lookup",buffImgSource,table);
		buffImgDest = imgDest.getAsBufferedImage();
			
		return buffImgDest;
	}	
	
	
	

	
	public static BufferedImage substractImage(BufferedImage buffImg1, BufferedImage buffImg2){
		BufferedImage buffImgDest;
		/*
		ParameterBlock pb;
		pb = new ParameterBlock();
		pb.addSource(buffImg1);
		pb.addSource(buffImg2);
		PlanarImage sub = (PlanarImage)JAI.create("Subtract", pb);    
		buffImgDest = sub.getAsBufferedImage();
		*/
		return buffImg1;
	}
	
/*
	public static BufferedImage substractImage(BufferedImage buffImg1, BufferedImage buffImg2){
		BufferedImage buffImgDest;
		
		ParameterBlock pb;/*
		pb = new ParameterBlock();
		pb.addSource(buffImg1);
		RenderedImage src1 = (RenderedImage)JAI.create("stream",pb);
		
		pb = new ParameterBlock();
		pb.addSource(buffImg2);
		RenderedImage src2 = (RenderedImage)JAI.create("stream",pb);
		pb = new ParameterBlock();
		pb.addSource(src1);
		pb.addSource(src2);
		pb.addSource(buffImg1);
		pb.addSource(buffImg2);
		//PlanarImage imgDest = (PlanarImage)JAI.create("Xor",pb);	
		PlanarImage sub = JAI.create("Subtract", pb);    
		PlanarImage dif = JAI.create("Absolute", sub);
		//RenderedOp dst = JAI.create("Subtract",pb);	
		//PlanarImage imgDest = dst.createInstance();
		
		/*RenderedImage dst = (RenderedImage)JAI.create("Subtract",pb);	
		Raster pixels = dst.getData();
		DataBuffer buff = pixels.getDataBuffer();
		int[] pixelTab  = new int[buff.getSize()];
		for (int i=0;i<buff.getSize();i++){
			int pixelColor = buff.getElem(i);
			pixelTab[i] = pixelColor;
		}
			JPanel temp =new JPanel();
		Image image2 = temp.createImage(new MemoryImageSource(320, 240, pixelTab, 0, 320));
		buffImgDest = dif.getAsBufferedImage();
		
		return buffImg1;
	}*/
	
	/*
	public void seuiller(double lowR, double highR, double lowG,double highG, double lowB,double highB){
		double[] low, high, map;
		low = new double[3];
		high = new double[3];
		map = new double[3];
		
		low[0] = lowR;
		high[0] = highR;
		map[0] = 255.0;
		low[1] = lowG;
		high[1] = highG;
		map[1] = 255.0;
		low[2] = lowB;
		high[2] = highB;
		map[2] = 255.0;
		
		ParameterBlock pb = new ParameterBlock();
		pb.addSource(im);
		pb.add(low);
		pb.add(high);
		pb.add(map);
		
		PlanarImage dst = JAI.create("threshold",pb);
		mainBuffImg = dst.getAsBufferedImage();
	}
	*/
	
	
	/*
	public static BufferedImage gammaCorrection(BufferedImage buffImgSource, float gamma) {
		PNGEncodeParam PNGEP = PNGEncodeParam.getDefaultEncodeParam(buffImgSource);
		PNGEP.setGamma(gamma);
		return buffImgSource;
	}
	*/
}