// Updates: 2002.06.02


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;


public class Contest extends JFrame implements ActionListener {
  Simulator simulator;
  AI ai;
  Tower tower;
  TerrainDistributor terrainDistributor;
  JTextArea ta;
  JTextFieldX tf;
  JButton update, restart, next, go;
  JCheckBox towerCB, camCB;
  
    
  public Contest() {
    super("   Contest");
        
    LookAndFeelMenu lafm = new LookAndFeelMenu(this, true);
    lafm.addPlaf("Kunststoff", "com.incors.plaf.kunststoff.KunststoffLookAndFeel");
    lafm.setLaf("Kunststoff");   
        
    JPanel p = new JPanel();
    p.setLayout(new BorderLayout());
    simulator = new Simulator();
    p.add(simulator, BorderLayout.CENTER);
    
    JPanel p0 = new JPanel();
    p0.setBorder(BorderFactory.createLineBorder(Color.black, 3));
    Box box = Box.createVerticalBox();
    ta = new JTextArea(6,10);
    ta.setEditable(false);
//    box.add(ta);

    p.add(box, BorderLayout.SOUTH);
   
    JPanel p1 = new JPanel();
    update = new JButton("Update");
    update.addActionListener(this);
    update.doClick();
//    p1.add(update);
    towerCB = new JCheckBox("Tower");
    towerCB.addActionListener(this);
    p1.add(towerCB);
    camCB = new JCheckBox("Camera");
    camCB.addActionListener(this);
    p1.add(camCB);    
    box.add(p1);
    
    JPanel p2 = new JPanel();
    restart = new JButton("Restart AI");
    restart.addActionListener(this);
    p2.add(restart);    
    next = new JButton("Next");
    next.addActionListener(this);
    p2.add(next);
    go = new JButton("Go !");
    go.addActionListener(this);
    p2.add(go);    
    p2.add(new JLabel("       Command: "));
    tf = new JTextFieldX(7);
    tf.addActionListener(this);
    p2.add(tf);
    box.add(p2);

    p0.add(box);
    p.add(p0, BorderLayout.SOUTH);  
    getContentPane().add(p);

    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        System.exit(0);
      }
    });
    pack();
    Dimension d1 = getToolkit().getScreenSize();
    Dimension d2 = this.getSize();     
    setLocation(d1.width/2 - d2.width/2, d1.height/2 - d2.height/2);
    SwingUtilities.updateComponentTreeUI(this);    
    show();
    
    ai = new AI();
  }


  public void actionPerformed(ActionEvent e) {
    Object source = e.getSource();
    if (source.equals(update)) update();
    else if (source.equals(restart)) ai = new AI();
    else if (source.equals(next)) next();
    else if (source.equals(go)) go();
    else if (source.equals(towerCB)) towerOnOff();
    else if (source.equals(camCB)) camOnOff();
    else if (source.equals(tf)) execute(tf.getText());
    repaint();
  }


  public static void main(String[] args) {
    new Contest();
  }
  
  
  private void update() {
    Point p = simulator.getHorizontalDoorPos();
    ta.setText("  horizontal door =  (" + p.x + "," + p.y + ")\n");
    p = simulator.getVerticalDoorPos();
    ta.append("  vertical door =  (" + p.x + "," + p.y + ")\n"); 
    p = simulator.getBlueCylinderPos();
    ta.append("  blue cylinder =  (" + p.x + "," + p.y + ")\n"); 
    p = simulator.getGreenCylinderPos();
    ta.append("  green cylinder =  (" + p.x + "," + p.y + ")\n"); 
    p = simulator.getRedCylinderPos();
    ta.append("  red cylinder =  (" + p.x + "," + p.y + ")\n"); 
    Triplet t = simulator.getRCXPos();
    ta.append("  RCX =  (" + t.x + "," + t.y + "," + t.a + ")"); 
  }
  
  
  private void next() {
    Positions pos = simulator.getAllPositions();
    String command = ai.next(pos);
    System.out.println("AI Command: " + command);    
    execute(command);
  }
    
    
  private void go() {
    String command = "";
    do {
      Positions pos = simulator.getAllPositions();
      command = ai.next(pos);
      System.out.println("AI Command: " + command);  
      execute(command);      
    } while(!command.equals("" + GC.FINISHED));
    System.out.println("Finished");
  }
  

  private void towerOnOff() {
    if (towerCB.isSelected()) tower = new Tower();
    else {
      tower.close();
      tower = null;
    }
  }
  
  
  private void camOnOff() {
    if (camCB.isSelected()) terrainDistributor = new TerrainDistributor();
    else terrainDistributor = null;
  }


  private void execute(String cmd) {    
    boolean tow = towerCB.isSelected();
    boolean cam = camCB.isSelected();
    if (cmd.indexOf(" ") == -1) cmd += " 0";

    if ((!tow) && (!cam)) {
      simulator.execute(cmd);
    }
    if ((tow) && (!cam)) {
      tower.execute(cmd);
      simulator.execute(cmd);
    }    
    if ((!tow) && (cam)) {
      simulator.setAllPositions(terrainDistributor.getPositions());
    }   
    if ((tow) && (cam)) {
      tower.execute(cmd);
      simulator.setAllPositions(terrainDistributor.getPositions());
    }      

    if (cmd.startsWith(""+GC.CMD_TAKE_CYL)) {
      simulator.cylinderIsMoving(ai.getCurrentCylinderIndex());
    }
    if (cmd.startsWith(""+GC.CMD_DROP_CYL)) {
      simulator.cylinderIsInGoal(ai.getCurrentCylinderIndex());
    }      
    update.doClick();
  }
}