// Updates: 2002.06.10


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.geom.*;


public class Simulator extends JPanel implements ActionListener, MouseListener, MouseMotionListener {
  final int width = 840;
  final int height = 1188;
  final double zoom = 0.5;
      
  JPopupMenu popup;
  int mouseX, mouseY;

  Goal blueGoal = new Goal(0, 685, 332); 
  Goal greenGoal = new Goal(1, 306, 258); 
  Goal redGoal = new Goal(2, 560, 166); 
  Door horizontalDoor = new Door();
  Door verticalDoor = new Door(horizontalDoor);
  Cylinder blueCylinder = new Cylinder(0);
  Cylinder greenCylinder = new Cylinder(1);
  Cylinder redCylinder = new Cylinder(2);
  RCX rcx = new RCX();


  public Simulator() {
    setPreferredSize(new Dimension((int)(width*zoom), (int)(height*zoom)));
    setMaximumSize(new Dimension((int)(width*zoom), (int)(height*zoom)));
    addMouseListener(this);
    addMouseMotionListener(this);
    popup = new JPopupMenu();
    buildPopupMenu(popup, new String[] {"Put horizontal door", "Put vertical door", "-", "Put blue cylinder", "Put green cylinder", "Put red cylinder", "-", "Put RCX", "-", "Set default config"});
    setDefaultConfig();
  }


  public Point getHorizontalDoorPos() {
    return horizontalDoor.getPosition();
  }
  
  
  public Point getVerticalDoorPos() {
    return verticalDoor.getPosition();
  }
  
  
  public Point getBlueCylinderPos() {
    return blueCylinder.getPosition();
  }
   

  public Point getGreenCylinderPos() {
    return greenCylinder.getPosition();
  }
  
  
  public Point getRedCylinderPos() {
    return redCylinder.getPosition();
  }    
  
  
  public Point getBlueGoalPos() {
    return blueGoal.getPosition();
  }  
  
  
  public Point getGreenGoalPos() {
    return greenGoal.getPosition();
  }  
  
  
  public Point getRedGoalPos() {
    return redGoal.getPosition();
  }
  
    
  public Triplet getRCXPos() {
    return rcx.getPosition();
  }  


  public Positions getAllPositions() {
    Positions positions = new Positions();
    positions.horizontalDoor = getHorizontalDoorPos();
    positions.verticalDoor = getVerticalDoorPos();
    positions.blueCylinder = getBlueCylinderPos();
    positions.greenCylinder = getGreenCylinderPos();
    positions.redCylinder = getRedCylinderPos();
    positions.blueGoal = getBlueGoalPos();
    positions.greenGoal = getGreenGoalPos();
    positions.redGoal = getRedGoalPos();
    positions.rcx = getRCXPos();
    return positions;
  }
    
  
  public void setAllPositions(Positions pos) {
    horizontalDoor.setPosition(pos.horizontalDoor);
    verticalDoor.setPosition(pos.verticalDoor);
    blueCylinder.setPosition(pos.blueCylinder);
    greenCylinder.setPosition(pos.greenCylinder);
    redCylinder.setPosition(pos.redCylinder);   
    blueGoal.setPosition(pos.blueGoal);
    greenGoal.setPosition(pos.greenGoal);
    redGoal.setPosition(pos.redGoal);  
    rcx.setPosition(pos.rcx);
    repaint();
  } 
  
   
  public void execute(short angle, short dist) {
    if (angle == GC.CMD_TAKE_CYL) rcx.takeCylinder();
    else if (angle == GC.CMD_DROP_CYL) rcx.dropCylinder();
    else if (angle > 500) return;
    else {
      rcx.rotate((int)angle);
      rcx.travel((int)dist);
    }
    repaint();
  }
  
  
  public void execute(String cmd) {
    String sAngle = cmd.substring(0, cmd.indexOf(" "));
    String sDist = cmd.substring(cmd.indexOf(" ") + 1, cmd.length());
    short angle = Short.parseShort(sAngle);
    short dist = Short.parseShort(sDist);
    execute(angle, dist);
  }
        
        
  public void cylinderIsMoving(int index) {
    if (index == 0) {
      blueCylinder.setState(Cylinder.NULL);
      rcx.carry(Color.blue);
    }
    if (index == 1) {
      greenCylinder.setState(Cylinder.NULL);
      rcx.carry(Color.green);
    }
    if (index == 2) {
      redCylinder.setState(Cylinder.NULL);
      rcx.carry(Color.red);
    }
  }
  

  public void cylinderIsInGoal(int index) {
    if (index == 0) blueCylinder.setPosition(blueGoal.getX(), blueGoal.getY());
    if (index == 1) greenCylinder.setPosition(greenGoal.getX(), greenGoal.getY());
    if (index == 2) redCylinder.setPosition(redGoal.getX(), redGoal.getY());
    rcx.carry(Color.black);
  }
  

  private void buildPopupMenu(JPopupMenu popup, String[] items) {
    JMenuItem mi;
    for (int i = 0; i < items.length; i++) {
      if (items[i].equals("-")) {
        popup.addSeparator();
      }
      else {
        mi = new JMenuItem(items[i]);
        mi.addActionListener(this);
        popup.add(mi);
      }
    }
  }
  
  
  private void setDefaultConfig() {    
    horizontalDoor.setPosition(292, 496);
    verticalDoor.setPosition(494, 870);
    blueCylinder.setPosition(134, 982);
    greenCylinder.setPosition(374, 806);
    redCylinder.setPosition(720, 650);     
    rcx.setPosition(100, 108, 0);
    rcx.carry(Color.black);
  }            

  
  public void paintComponent(Graphics g) {
    super.paintComponent(g); // draw the background
    Graphics2D g2 = (Graphics2D)g;
    RenderingHints hints = new RenderingHints(null);
    hints.put(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    g2.setRenderingHints(hints);
    g2.setTransform(new AffineTransform(zoom,0,0,-zoom,0,height*zoom));

    g.drawLine(0, 250, 200, 250);
    g.drawLine(200, 250, 200, 0);

    blueGoal.paint(g);
    greenGoal.paint(g);
    redGoal.paint(g);

    if (horizontalDoor.isPlaced()) horizontalDoor.paint(g);
    if (verticalDoor.isPlaced())   verticalDoor.paint(g);
    if (blueCylinder.isPlaced())   blueCylinder.paint(g);
    if (greenCylinder.isPlaced())  greenCylinder.paint(g);
    if (redCylinder.isPlaced())    redCylinder.paint(g);
    if (rcx.isPlaced())            rcx.paint(g);

    if (horizontalDoor.isPlacing()) horizontalDoor.paint(g, mouseX, mouseY);
    if (verticalDoor.isPlacing())   verticalDoor.paint(g, mouseX, mouseY);
    if (blueCylinder.isPlacing())   blueCylinder.paint(g, mouseX, mouseY);
    if (greenCylinder.isPlacing())  greenCylinder.paint(g, mouseX, mouseY);
    if (redCylinder.isPlacing())    redCylinder.paint(g, mouseX, mouseY);
    if (rcx.isPlacing())            rcx.paint(g, mouseX, mouseY);

    g.setColor(Color.black);
    g2.setTransform(new AffineTransform());
    g.drawString("("+mouseX+","+mouseY+")", (int)(width*zoom-130*zoom), (int)(height*zoom-5));
  }
  
 
  public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();
    if (cmd.equals("Put horizontal door")) horizontalDoor.setState(Door.PLACING);
    if (cmd.equals("Put vertical door"))   verticalDoor.setState(Door.PLACING);
    if (cmd.equals("Put blue cylinder"))   blueCylinder.setState(Cylinder.PLACING);
    if (cmd.equals("Put green cylinder"))  greenCylinder.setState(Cylinder.PLACING);    
    if (cmd.equals("Put red cylinder"))    redCylinder.setState(Cylinder.PLACING);         
    if (cmd.equals("Put RCX"))             rcx.setState(RCX.PLACING);
    if (cmd.equals("Set default config"))  setDefaultConfig();
    repaint();
  }
    
 
  public void mouseMoved(MouseEvent e) {
    mouseX = (int)(e.getX()/zoom);
    mouseY = (int)(height-e.getY()/zoom);
    repaint();
  }
  
  
  public void mouseReleased(MouseEvent e) {
    if (e.isPopupTrigger()) popup.show(e.getComponent(), e.getX(), e.getY());
  }  
  
 
  public void mousePressed(MouseEvent e) {
    int x = (int)(e.getX()/zoom);
    int y = (int)(height-e.getY()/zoom);
    if (horizontalDoor.isPlacing()) horizontalDoor.setPosition(x, y);
    if (verticalDoor.isPlacing())   verticalDoor.setPosition(x, y);
    if (blueCylinder.isPlacing())   blueCylinder.setPosition(x, y);
    if (greenCylinder.isPlacing())  greenCylinder.setPosition(x, y);
    if (redCylinder.isPlacing())    redCylinder.setPosition(x, y);
    if (rcx.isPlacing()) {
      int alpha = 0;
      String inputAlpha = JOptionPane.showInputDialog("Alpha [0..359] = ");
      try {
        alpha = Integer.parseInt(inputAlpha) % 360;
      } catch (NumberFormatException e2) {} 
      rcx.setPosition(x, y, alpha);
    }            
    repaint();
  }
  
  
  public void mouseClicked(MouseEvent e) {}
  public void mouseEntered(MouseEvent e) {}
  public void mouseExited(MouseEvent e) {}  
  public void mouseDragged(MouseEvent e) {}
}